﻿//*****************************************************************************
// Description     : Main sample program to measure histogram
// Target Device   : APU101
// Target Platform : Windows, .NET Framework, C++
// Date            : July 2017
// Version         : 1.0
//*****************************************************************************
#include <Winsock2.h>
#include "file.h"
#include "sitcp.h"
#include "device.h"
#include "local_define.h"

using namespace System::Runtime::InteropServices;
using namespace System::Threading;


//
// Local functions declaration
//
static int config_device(SiTCPcom^, unsigned qword);
static int run_measurement(SiTCPcom^, unsigned qword);
static int receive_data(SiTCPcom^);


//#########################################################################
//  Name      : main
//  Function  : entry point
//  Arguments :
//      in args : parameters
//  Return :
//      0 : success
//      1 : failure
//  Note : 
//#########################################################################
int main(array<System::String ^> ^args)
{
    int run_ready;
    int ret;
    DEVICE_COM_INFO ^comInfo = gcnew DEVICE_COM_INFO;
    unsigned qword meas_count;

    run_ready = 0;

    // 1. connect to the device
    ret = device_connect(comInfo);

    if (ret != 0) {
        return(1);
    }

    // 2. set parameters to the device
    meas_count = device_msec2count(MEASUREMENT_TIME);   // convert msec to count
    ret = config_device(comInfo, meas_count);
    if (ret == 0) {
        run_ready = 1;
    } else {
        Console::WriteLine("setting device config error({0})...", ret);
    }

    if (run_ready == 1) {   // ready to run
        // 3. start measurement
        Console::WriteLine("start measurement!!!");
        ret = run_measurement(comInfo, meas_count);

        Console::WriteLine("finish measurement...");

        if (ret >= 0) {
            Console::WriteLine("measurement completed!!!");
        }
        else {
            Console::WriteLine("measurement error({0})...\n", ret);
        }
    }

    // 4. post processing
    device_disconnect(comInfo);

    return(0);
}


//#########################################################################
//  Name      : config_device
//  Function  : Set parameters to device
//  Arguments :
//      in comInf   : instance of DEVICE_COM_INFO class
//      in ms_count : measurement time (counts)
//  Return :
//       0 : success
//      -n : failed at nth parameter
//  Note : 
//#########################################################################
static int config_device(DEVICE_COM_INFO ^comInf, unsigned qword ms_count)
{
    int ret;

    // polarity: positive
    ret = device_write_16bit(comInf, DEVICE_COMMAND_POL, DEVICE_PARAM_POL_POS);
    if (ret < 0)  { return(-1); }

    // analog coarse gain: x5
    ret = device_write_16bit(comInf, DEVICE_COMMAND_ACG, DEVICE_PARAM_ACG_x5);
    if (ret < 0)  { return(-2); }

    // ADC gain: 4096
    ret = device_write_16bit(comInf, DEVICE_COMMAND_ADG, DEVICE_PARAM_ADG_4096);
    if (ret < 0)  { return(-3); }

    // fast trigger threshold: 30
    ret = device_write_16bit(comInf, DEVICE_COMMAND_FTH, 30);
    if (ret < 0)  { return(-4); }

    // fast filter pole zero: 0
    ret = device_write_16bit(comInf, DEVICE_COMMAND_FPZ, 0);
    if (ret < 0)  { return(-5); }

    // slow filter rise time: 1000ns(100 @1digit=10ns)
    ret = device_write_16bit(comInf, DEVICE_COMMAND_SFR, 100);
    if (ret < 0)  { return(-6); }

    // slow filter peaking time: 1100ns(110 @1digit=10ns)
    ret = device_write_16bit(comInf, DEVICE_COMMAND_SFP, 110);
    if (ret < 0)  { return(-7); }

    // slow filter pole zero: 660ns(66 @1digit=10ns)
    ret = device_write_16bit(comInf, DEVICE_COMMAND_SPZ, 66);
    if (ret < 0)  { return(-8); }

    // slow filter threshold: 40
    ret = device_write_16bit(comInf, DEVICE_COMMAND_STH, 40);
    if (ret < 0)  { return(-9); }

    // digital coarse gain: x16
    ret = device_write_16bit(comInf, DEVICE_COMMAND_DCG, DEVICE_PARAM_DCG_x16);
    if (ret < 0)  { return(-10); }

    // digital fine gain: x0.5 (8193 x 0.5 - 2 = 4094)
    ret = device_write_16bit(comInf, DEVICE_COMMAND_DFG, 4094);
    if (ret < 0)  { return(-11); }

    // reset filter: 0,1,0
    ret = device_write_3step(comInf, DEVICE_COMMAND_RFL);
    if (ret < 0)  { return(-12); }

    // DAC monitor: slow
    ret = device_write_16bit(comInf, DEVICE_COMMAND_DAC, DEVICE_PARAM_DAC_SLOW);
    if (ret < 0)  { return(-13); }

    // measurement mode: histogram
    ret = device_write_16bit(comInf, DEVICE_COMMAND_MOD, DEVICE_PARAM_MOD_HIST);
    if (ret < 0)  { return(-14); }

    // measurement time
    ret = device_write_48bit(comInf, DEVICE_COMMAND_MTM, ms_count);
    if (ret < 0)  { return(-15); }

    return(0);
}


//#########################################################################
//  Name      : run_measurement
//  Function  : Measurement processing
//  Arguments :
//      in comInf   : instance of DEVICE_COM_INFO class
//      in ms_count : measurement time (counts)
//  Return :
//       0 : success
//      -1 : failed while measurement
//      -2 : failed to write file header
//      -3 : failed to receive histogram data
//  Note : 
//#########################################################################
static int run_measurement(DEVICE_COM_INFO ^comInf, unsigned qword ms_count)
{
    int ret;
    unsigned qword now_count;
    unsigned int now_msec;
    unsigned short status;
    unsigned int input_cps;
    unsigned int throughput_cps;

    // clear data
    device_write_3step(comInf, DEVICE_COMMAND_CLR);
    // wait for completion of clear processing
    Thread::Sleep(100);

    // start measurement
    device_start_measurement(comInf);

    while(1) {
        // read status
        ret = device_read_status(comInf, &status, &now_count, &input_cps, &throughput_cps);

        if (ret < 0) { // communication error
            // quit measurement
            break;
        }
        // status is 'stopped' or measurement time elapsed
        if ((status == DEVICE_PARAM_AQS_STOP) || (now_count >= ms_count)) {
            break;
        }

        Thread::Sleep(200);   // wait for 200ms
    }

    // stop measurement
    device_stop_measurement(comInf);

    // quit by error
    if (ret < 0) {
        return(-1);
    }

    // write file header
    now_msec  = (unsigned int)(device_count2msec(now_count));    // convert count to msec
    ret =  file_write_header(SAVE_FILENAME, now_msec, input_cps, throughput_cps);
    if (ret < 0) {
        return(-2);
    }

    // receive histogram data then write out to file
    ret = receive_data(comInf);
    if (ret < 0) {
        return(-3);
    } else {
        return(0);
    }
}


//#########################################################################
//  Name      : receive_data
//  Function  : receiving collected data
//  Arguments :
//      in comInf : instance of DEVICE_COM_INFO class
//  Return :
//       0 : success
//      -1 : failed to receive data
//      -2 : failed to write file
//  Note : 
//#########################################################################
static int receive_data(DEVICE_COM_INFO ^comInf)
{
    int ret;
    int result;
    unsigned int hdata[8192];  // for APU101
//  unsigned int hdata[4096];  // for APU101X

    result = 0;

    // receive histogram data
    memset(hdata, 0, sizeof(hdata));
    ret = device_read_histogram(comInf,
                               (char*)(hdata), sizeof(hdata),
                                DEVICE_TIMEOUT_DATA);

    if (ret > 0) {
        // write data to file
        ret =  file_write_data(SAVE_FILENAME, sizeof(hdata), (void *)hdata);
        if (ret < 0) {
            // failed to write
            result = -2;
        }
    } else {
        // failed to receive data
        result = -1;
    }

    return(result);
}
